/* 
   dlanconfig.c: dlanconfig tool for HomePlug devices for linux

   Copyrighted 2004 by devolo AG
   Only for use with MicroLink dLAN adaptors from devolo. 

   Redistribution in source form, with or without modification,
   are NOT permitted.
   This program uses explicitly confidential information
   about HomePlug standard.
*/

#include "md5.h" 
#include "dlanconfig.h"

#include <signal.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <stdio.h>
#include <sys/uio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <errno.h>
#include <arpa/inet.h>
#include <netinet/if_ether.h> 
#include <unistd.h>
#include <string.h>
#include <sys/mman.h> 
#include <linux/if_packet.h> 
#include <ctype.h>
#include <wait.h>

/*
 *  Define
 */
#define LOCAL  '1'
#define REMOTE '2'
#define POLL '3'
#define EXIT '4'
#define GETLOCMAC  '5'

#define EXITstr   "Exit"
#define POLLstr   "List remote devices"
#define LOCALstr  "Set local network password"
#define REMOTEstr "Set remote network password"
#define PRINTOPT(a,b)  printf("%c) %s\n",a,b)
#define DOWN 1
#define UP   0

// number of retry to get localMAC
#define MAXRETRY 5

/*
 *  Global variables
 */
static devolo_table_t* p_devTable=NULL; 
static char localIfName[MAX_INAME_LEN];
static unsigned char localMAC[MACSIZE];
static char ifstatus=UP;


/*
 *   Prototypes
*/

int   ListenForPackets(int fileDescr,char * buffOut,int maxNumber,int maxTimeS);
inline int   IsSameMAC(unsigned char* mac1,unsigned char* mac2);
inline int   IsAlmostSameMAC(const void* mac1,const void* mac2); 
void  Dump_NetRespPkt(void* packet);
void  GetLocalParExtResp(void* buffer );
int   SendSetLocalNek( char *iname,unsigned char* EK_str); 
int   SendSetRemoteNek(char *iname,char* EK_str,unsigned char* remotePasswd);
void   GetUpdateDevTable(char*  mac);
void  GetUpdatePhyDataRates(void* packet);
inline void  PrintMAC(unsigned char* mac_addr);
void  InitNode();
int   WaitForNbytes(int fd,int expectedBytes,unsigned char* buf,struct timeval tv,int extraTimeSec);
int   GetNextPacket(int fileDescr,void *strPacket);
int   DoMain(char action,char* iname,char* newNek,char* remotePasswd);
void ifdown(char *);

/*
 * Functions
 */

/*
 *  Update device table with a new mac
 */
void GetUpdateDevTable(char*  mac){
  int i=0;

  if(IsSameMAC(mac,p_devTable->locBridge.MAC))
    return;

  for(i=0;i<p_devTable->devCounter;i++)
    if(IsSameMAC(p_devTable->list[i].MAC,mac))
      return;
  
  memcpy(p_devTable->list[p_devTable->devCounter].MAC,mac,MACSIZE);
  p_devTable->devCounter++;
  return;
}


/*
 *  Initialize struct of network devices
 */
void InitNode(){
  memset(p_devTable,0x0,sizeof(devolo_node_t)*MAX_NET_INTERFACE_NUM);
  p_devTable->devCounter=0;
  p_devTable->locBridge.devType=-1; 
}


void ifdown(char *name){
  int fd;
  struct ifreq ifr;
    fd = socket(AF_INET, SOCK_DGRAM, 0);

    printf("\nbring %s down...\n",name);
    if(fd == -1){
      perror("error opening socket");
      return;
    }
    memset(&ifr, 0, sizeof(struct ifreq));
    strncpy(ifr.ifr_name, name, IF_NAMESIZE);
    ifr.ifr_flags &= (short int)~IFF_UP;  
    if(ioctl(fd, SIOCSIFFLAGS, &ifr) <0)
      perror("Error setting flag");
}   

/*
  checks for the iff_up flag and then bring interface up
  if necessary. brings interface MAC 
 */
int InitIf(char *name){

  int fd;
  struct ifreq ifr;
  fd = socket(AF_INET, SOCK_DGRAM, 0);
  
  if(fd == -1){
    perror("error opening socket");
    return 1;
  }
    
  memset(&ifr, 0, sizeof(struct ifreq));
  strncpy(ifr.ifr_name, name, IF_NAMESIZE);
  
  if(ioctl(fd, SIOCGIFFLAGS, &ifr)<0){
    /* necessary to let kernel modprobe driver
     and eventually ifup the interface */
    sleep(1);     
    if(ioctl(fd, SIOCGIFFLAGS, &ifr)<0){
      perror(NULL);
      close(fd);
      return 1;
    }
  }


  if ((ifr.ifr_flags & IFF_UP) == 0){
    ifstatus=DOWN;
    printf("bringing %s up..\n",name);
    ifr.ifr_flags |= IFF_UP;
    //ifup interface
    if(ioctl(fd, SIOCSIFFLAGS, &ifr) <0){
      perror("\nCannot ifup");
      close(fd);
      return 1;
    }
  }
  
  //get mac address
  if ( ioctl(fd,SIOCGIFHWADDR, &ifr) == -1 ){
    perror("\nCannot register mac address");
    close(fd);
    return 1;
  }
  
  memcpy(localMAC,ifr.ifr_hwaddr.sa_data,MACSIZE);
  close(fd);
  return 0;
}



/*
 *  Print a formatted version of MAC address
 */
inline void PrintMAC(unsigned char *mac_addr) 
{ 
  unsigned char* mac;
  for(mac=mac_addr;mac<mac_addr+MACSIZE-1;mac++)
    printf("%02X:",*mac);
  printf("%02X",*mac);
  fflush(stdout);
}


/*
 *   Check if two MAC address are egual
 */
inline int IsSameMAC(unsigned char* mac1,unsigned char* mac2)
{
  int i;
  for(i=0;i<MACSIZE;i++)
    if(mac1[i] != mac2[i])
      return 0;
  return 1;
}


/*
 *   Send a packet to `iname' network interface
 */
int SendPacket(char *iname,unsigned char addr[],void *buffer,size_t len)
{
  int i=0;
  int retval=0;
  struct sockaddr_ll destAddr;
  struct sockaddr_ll sourceAddr;
  int sk=0;
  struct ifreq ifr;

  memset(&sourceAddr, 0, sizeof(sourceAddr));
  memset(&destAddr, 0, sizeof(destAddr));
  strcpy(ifr.ifr_name,iname);
  
  sk = socket(PF_PACKET, SOCK_DGRAM, htons(SOCK_PROTO));
  if ( sk < 0 ){
      printf("[ERROR]: SendPacket: socket unable to send packet \n");
      return -1;
  }

  if (ioctl(sk,SIOCGIFINDEX,&ifr)){
    printf("[ERROR]: SendPacket: error ioctl SIOCGIFINDEX errno=%d\n",errno);
    close(sk);
    return -1;
  }

  destAddr.sll_family = AF_PACKET;
  destAddr.sll_protocol =htons(ETHERTYPE_INTELLON); 
  destAddr.sll_ifindex= ifr.ifr_ifindex;             
  destAddr.sll_halen = 6;
  
  for(i=0;i<MACSIZE;i++)
    destAddr.sll_addr[i]=addr[i];
    
  sourceAddr.sll_family = PF_PACKET;
  sourceAddr.sll_protocol=htons(SOCK_PROTO);
  sourceAddr.sll_ifindex = ifr.ifr_ifindex;

  retval = bind(sk,(struct sockaddr *) &sourceAddr, sizeof(sourceAddr));
  if ( retval==-1){
      perror("SendPacket: bind error\n");
  }else
    retval = sendto(sk, buffer, len, 0,(struct sockaddr *) &destAddr,sizeof destAddr);
  if ( retval==-1)
    perror("SendPacket: sendto error\n");
  
  close(sk);
  return(retval);
}



/*
 *   Send a packet to `iname' network interface
 *   for a network stats request
 */
int SendNetStatReq( char *iname ){  
  unsigned char mac_dest_addr[MACSIZE];      
  unsigned char outFrame[MME_NET_STAT_FRAME_LEN-MME_HDR_LEN];
  int retval=0;
  
  memset( outFrame,0x00, MME_NET_STAT_FRAME_LEN-MME_HDR_LEN); 
  memset(mac_dest_addr,0xff,MACSIZE);  
  
  outFrame[0] = 0x01;	           
  outFrame[1] = CODE_ReqNetStats;  
  outFrame[2] = 0xbb;              
  outFrame[3] = 0x80;              
  
  retval = SendPacket(iname,mac_dest_addr,outFrame,MME_NET_STAT_FRAME_LEN - MME_HDR_LEN);  
  return(retval);
}


/*
 *   Send a packet to `iname' network interface
 *   for a stats request
 */
int SendStatReq( char *iname ){
  unsigned char mac_dest_addr[MACSIZE];
  unsigned char outFrame[MME_MIN_FRAME_LEN-MME_HDR_LEN];
  int retval=0;
  
  memset(outFrame,0x00, MME_MIN_FRAME_LEN-MME_HDR_LEN);
  memset(mac_dest_addr,0xff,MACSIZE);  
	  
  outFrame[0] = 0x01;	          
  outFrame[1] = CODE_ReqStats;    
  outFrame[2] = MME_ReqStatsSize; 
  outFrame[3] = 0x0;              
  
  retval=SendPacket(iname,mac_dest_addr,outFrame,MME_MIN_FRAME_LEN-MME_HDR_LEN);  
  return(retval);
}


/*
 *   Send a packet to `iname' network interface
 *   for network statistics request
 */
int SendNetworkStatBasic( char *iname){  
  unsigned char mac_dest_addr[MACSIZE]; 
  struct mme_hdr* pHdr;
  int retval=0;
  
  pHdr = (struct mme_hdr*) malloc(MME_NET_STAT_FRAME_LEN-MME_HDR_LEN);
  
  memset(mac_dest_addr,0xff,MACSIZE);
  memset(pHdr,0x00,MME_NET_STAT_FRAME_LEN-MME_HDR_LEN);
  
  pHdr->MCTRL =   0x01 ;         
  pHdr->MEHDR =   CODE_ReqNetStats;
  pHdr->MELEN =   0xbb;            
  pHdr->NETW_CTRL = 0x80 & 0x7F;   

  retval=SendPacket(iname,mac_dest_addr,pHdr,MME_NET_STAT_FRAME_LEN - MME_HDR_LEN);
  free(pHdr);  
  return retval;	  
}


/*
 *   Send a packet to `iname' network interface
 *   for local param request
 */
int SendLocalParExt( char *iname) {  
  unsigned char mac_dest_addr[MACSIZE];  
  struct mme_hdr* pHdr;
  int retval=0;
  
  pHdr = (struct mme_hdr*) malloc(MME_NET_STAT_FRAME_LEN-MME_HDR_LEN);
  memset(mac_dest_addr,0xff,MACSIZE); 
  memset(pHdr,0x00,MME_NET_STAT_FRAME_LEN-MME_HDR_LEN);
  
  pHdr->MCTRL = 0x1 ;                 
  pHdr->MEHDR = CODE_SetLocPar ;      
  pHdr->MELEN = MME_LocalParExtSize;  
  
  retval=SendPacket(iname,mac_dest_addr,pHdr,MME_NET_STAT_FRAME_LEN - MME_HDR_LEN);
  free(pHdr);
  return retval;	  
}


/*
 *   Parser of received packet 
 */
void PacketReceive_callback(char *packet)
{
  struct MME_PackedHdr* pHdr;

  if(packet == NULL)
    return;
  
  pHdr = (struct MME_PackedHdr*)packet;
  if(!IsSameMAC(pHdr->DA,localMAC) || pHdr->Ethertype!=htons(ETHERTYPE_INTELLON)) 
    return;

  if((pHdr->MEHDR == CODE_ReqStatsResp)) { 
    GetUpdateDevTable(pHdr->SA); 
    return; 
  }
    
  if( (pHdr->MEHDR == CODE_ReqNetStats)){
    GetUpdatePhyDataRates(packet);
    return;
  }
  
  if((pHdr->MEHDR == CODE_SetLocPar) && (pHdr->NETW_CTRL == 0x1)){
    GetLocalParExtResp(packet);
    return;
  }
  
}


/*
 *  Function that update physical 
 *  data rates
 */
void GetUpdatePhyDataRates(void* packet)
{
  struct API_ReqNetStatsResp* p;
  int    i,count;
  double rate=0.0;
  short  devType=0;
  
  p=(struct API_ReqNetStatsResp*)packet;
  
  devType=(short)p->MCTRL &  0x7F;
  if(devType>=0 && devType<=4)
    p_devTable->locBridge.devType=devType;
  else devType=-1; 

  for(i=0;i<p_devTable->devCounter;i++){   
    for( count=0; count<NWSIZE;count++ ){ 
      
      if(IsAlmostSameMAC(p_devTable->list[i].MAC,p->NW[count].DA)) {  
	rate = ((p->NW[count].BYTES40) / 42.0);
	rate = ( M_RATE * rate ) + B_RATE;
	p_devTable->list[i].rate=rate;
	
	if(IsSameMAC(p_devTable->list[i].MAC,p->NW[count].DA))
	  p_devTable->list[i].toggled=0;
	else
	  p_devTable->list[i].toggled=1;	 
	break;
      }
    }
  }
}

/*
 *  Checks if two MAC are equal apart from last bit 
 */
inline int IsAlmostSameMAC(const void* mac1,const void* mac2)
{
  char* macA;
  char* macB;
  int flag=0;

  macA=(char*)mac1;
  macB=(char*)mac2;
  
  flag=((macA[0]|0x01)==(macB[0]|0x01));  
  return( flag && macA[1]==macB[1] && macA[2]==macB[2] 
	  && macA[3]==macB[3] && macA[4]==macB[4] && macA[5]==macB[5]);
}



/*
 *  Function that print the list of 
 *  connected devices
 */
void PrintNode(char* iname)
{
  unsigned int i,j=1;
  double aggRate=0.0;  

  printf("\n");
  if(p_devTable->devCounter<=0){
    printf("Not found remote devices\n");
    return;
  }
 
  for(i=0;i<p_devTable->devCounter;i++)
    aggRate+=p_devTable->list[i].rate;
  aggRate/=p_devTable->devCounter;
 
  printf("Remote devices:\n");

  printf("\tNum.\tMAC address       \tMbps\tCache\n");
  for(i=0;i<p_devTable->devCounter;i++){

      printf("\t%d\t",j++);
      PrintMAC(p_devTable->list[i].MAC);
      printf("\t%1.3f",p_devTable->list[i].rate);
      if(p_devTable->list[i].toggled) 
	printf("\t  *");
      printf("\n");
  }
  printf("\n");
  printf("Physical data rate = %1.3f Mbps\n",aggRate);
  fflush(stdout);
}


/*
 *   Help
 */
void help(char *progname){
  printf("This tool allows you to manage an HomePlug network.\nIn order to use it you must be `root'.\nYou should also specify the local HomePlug interface you want to manage.\n\n");
  printf("usage: %s interface\n",progname);
  printf("example: %s dlanusb0\n\n",progname);
  printf("Read man pages for more information.\n");
}


/*
 *   Usage
 */
void usage(char *progname){
  printf("usage: %s interface\n",progname);
  printf("example: %s dlanusb0\n\n",progname);
  printf("Try '-h' or '--help' options, or read man page, for more information.\n");
}


/*
 *  Send packet to set local nek
 */
int SendSetLocalNek( char *iname,unsigned char* EK_str){  
  unsigned char mac_dest_addr[MACSIZE];  
  struct mme_double* pHdr; 
  unsigned char* pKey;
  int retval=0;
 
  pHdr=(struct mme_double*) malloc(MME_MIN_FRAME_LEN-MME_HDR_LEN);
  memset(mac_dest_addr,0xff,MACSIZE);
  memset(pHdr,0x00,MME_MIN_FRAME_LEN-MME_HDR_LEN) ; 
  
  pHdr->CTRL  = 0x2 ;   
  pHdr->HDR1  = 0x04;   
  pHdr->LEN1  = 0x09;   
  pHdr->EKS   = 0x01;   

  pKey = MD5HomePlugKey(EK_str);
  memcpy(pHdr->NEK,pKey,DECKSIZE);  
    
  pHdr->HDR2      = 0x1F;
  pHdr->LEN2      = 0x03;
  pHdr->TXCHAR[0] = 0x58;   
  pHdr->TXCHAR[1] = 0x84;   
  pHdr->TXCHAR[2] = 0x01;
   
  retval=SendPacket(iname,mac_dest_addr,pHdr,MME_MIN_FRAME_LEN-MME_HDR_LEN); 
  free(pHdr); 
  return retval;	  
}


/*
 *  Send packet to set remote nek
 */
int SendSetRemoteNek(char *iname,char* EK_str,unsigned char* remotePasswd){ 
  unsigned char mac_dest_addr[MACSIZE]; 
  struct mme_double* mme_d; 
  struct mme_triple* mme_tr;
  unsigned char* pKey;
  int retval=0;
 
  mme_d=(struct mme_double*) malloc(MME_MIN_FRAME_LEN);
  mme_tr=(struct mme_triple*) malloc(MME_MIN_FRAME_LEN);
  
  memset(mac_dest_addr,0xff,MACSIZE);
  memset(mme_d,0x00,MME_MIN_FRAME_LEN) ; 
  
  mme_d->CTRL  = 0x2 ;   
  mme_d->HDR1  = 0x04;   
  mme_d->LEN1  = 0x09;   
  mme_d->EKS   = 0x00;   

  pKey = MD5HomePlugKey(remotePasswd);
  memcpy(mme_d->NEK,pKey,DECKSIZE);  

  mme_d->HDR2      = 0x1F;
  mme_d->LEN2      = 0x03;
  mme_d->TXCHAR[0] = 0x58;   
  mme_d->TXCHAR[1] = 0x84;   
  mme_d->TXCHAR[2] = 0x00; 

  retval=SendPacket(iname,mac_dest_addr,mme_d,MME_MIN_FRAME_LEN-MME_HDR_LEN); 
  if(retval<=0){
    printf("[ERROR]: SendSetRemoteNek: sending first packet\n");
    free(mme_d);
    free(mme_tr);
    return(retval);
  }
  
  memset(mac_dest_addr,0xff,MACSIZE); 
  memset(mme_d,0x00,MME_MIN_FRAME_LEN - MME_HDR_LEN);

  mme_d->CTRL  = 0x01;  
  mme_d->HDR1  = 0x04;  
  mme_d->LEN1  = 0x09;  
  mme_d->EKS   = 0x01;  

  pKey = MD5HomePlugKey(EK_str); 
  memcpy(mme_d->NEK,pKey,DECKSIZE);  
  
  retval=SendPacket(iname,mac_dest_addr,mme_d,MME_MIN_FRAME_LEN-MME_HDR_LEN); 
  if(retval<=0){
    printf("[ERROR]: SendSetRemoteNek: sending second packet\n");
    free(mme_d);
    free(mme_tr);
    return(retval);
  }
  
  memset(mac_dest_addr,0xff,MACSIZE); 
  memset(mme_tr,0x00,MME_MIN_FRAME_LEN) ; 

  mme_tr->CTRL  = 0x03;   
  mme_tr->HDR1  = 0x04;   
  mme_tr->LEN1  = 0x09;   
  mme_tr->EKS1  = 0x00;   
  memcpy(mme_tr->NEK1,p_devTable->locBridge.DEK,DECKSIZE);

  mme_tr->HDR2  = 0x04; 
  mme_tr->LEN2  = 0x09; 
  mme_tr->EKS2  = 0x01;
  memcpy(mme_tr->NEK2,p_devTable->locBridge.NEK,DECKSIZE);

  mme_tr->HDR3  = 0x1F;
  mme_tr->LEN3  = 0x03;
  mme_tr->TXCHAR[0] = 0x58;  
  mme_tr->TXCHAR[1] = 0x84;  
  mme_tr->TXCHAR[2] = 0x01;
  
  retval=SendPacket(iname,mac_dest_addr,mme_tr,MME_MIN_FRAME_LEN-MME_HDR_LEN); 
  if(retval<=0){
    printf("[ERROR]: SendSetRemoteNek: sending third packet\n");
    free(mme_d);
    free(mme_tr);
    return(retval);
  }

  memset(mac_dest_addr,0xff,MACSIZE); 
  memset(mme_d,0x00,MME_MIN_FRAME_LEN-MME_HDR_LEN);
  mme_d->CTRL  = 0x2 ;  
  mme_d->HDR1  = 0x04;  
  mme_d->LEN1  = 0x09;  
  mme_d->EKS   = 0x01; 
  memcpy(mme_d->NEK,p_devTable->locBridge.NEK,DECKSIZE);
  
  mme_d->HDR2      = 0x1F;
  mme_d->LEN2      = 0x03;
  mme_d->TXCHAR[0] = 0x58;   
  mme_d->TXCHAR[1] = 0x84;   
  mme_d->TXCHAR[2] = 0x01;
   
  retval=SendPacket(iname,mac_dest_addr,mme_d,MME_MIN_FRAME_LEN-MME_HDR_LEN); 
  if(retval<=0){
   printf("[ERROR]: SendSetRemoteNek: sending fourth packet\n");
   free(mme_d);
   free(mme_tr);
   return(retval);
  }
  free(mme_d);
  free(mme_tr);
  return retval;
}


/*
 *   Parse local bridge parameter response
 */
void GetLocalParExtResp(void* packet)
{
  struct mme_locParExtResp* pHdr;
  pHdr = (struct mme_locParExtResp*) packet;

  memcpy(p_devTable->locBridge.MAC,pHdr->SA,MACSIZE);
  memcpy(p_devTable->locBridge.NEK,pHdr->NEK,DECKSIZE);
  memcpy(p_devTable->locBridge.DEK,pHdr->DEK,DECKSIZE);
}

/*
 *  Send packet to poll network
 */
int API_NetwPoll(char* iname)
{
  int count;
  int retval=0;
  
  for(count=0;count<POLL_RETRY;count++){
    retval+=SendStatReq(iname);
    sleep(INTER_FRAME_DELAY);
    retval+=SendNetStatReq(iname);
  }
  return retval;
}


/*
 *  Allocation of device table
 */
struct devolo_table* API_Alloc_DevTable(void)
{
  p_devTable= (struct devolo_table*) malloc(sizeof (devolo_table_t)*MAX_NET_INTERFACE_NUM);
 
  if (p_devTable == NULL) { 
    printf("[ERROR]: Alloc_DevTable: malloc error\n");
    strerror(errno);
    return NULL;
  }
 
  InitNode();       
  return(p_devTable);
}


/*
 * first checks
 */
inline int CHeckFirst(char *iname){

  if(InitIf(iname))
    return 1;
  
  if(!(p_devTable=API_Alloc_DevTable())){
    printf("[ERROR]: main: cannot allocate table memory\n");  
    return 1;
  }
  
  return 0;
}

/*
 *  Print main menu
 */
inline void printMenu(){
  printf(  "\nMain menu:\n");
  PRINTOPT(LOCAL,LOCALstr);
  PRINTOPT(REMOTE,REMOTEstr);
  PRINTOPT(POLL,POLLstr);
  PRINTOPT(EXIT,EXITstr);
  printf("\nChoose a command number: ");
}  


/*
 *  Main
 */
int main(int argc, char *argv[]){
  char iname[MAX_INAME_LEN]="";
  char buff[256];
  int cont=TRUE;

  printf("\n<-- dlanconfig tool v%s -->\n\n",DLANCONFIG_VERSION);
 
  /* usage case */ 
  if(argc<2) {
    usage(argv[0]); 
    return 0;
  }

  /* help case */ 
  if(!strcmp(argv[1],"-h") || !strcmp(argv[1],"--help")) {
    help(argv[0]); 
    return 0;
  }


  /* check root proviledges */
  if(getuid()!=0){
    printf("\nTo run dlanconfig tool you must have root priviledges.\n"); 
    fflush(stdout);  
    return 0; 
  }

  strcpy(iname,argv[1]);
  if(DoMain(GETLOCMAC,iname,NULL,NULL)){
    printf("\nsorry, exiting!\n");
    return 1;
  }

  while (cont){  
    printMenu();
    fgets((char *)buff,256,stdin);
    switch((char)buff[0]){
    case EXIT:
      if(ifstatus == DOWN)
	ifdown(iname);
      cont=FALSE;
      break;
    case POLL:
      {
	if(!DoMain(POLL,iname,NULL,NULL)){
	  printf("\nLocal dLAN device has MAC address ");
	  PrintMAC(p_devTable->locBridge.MAC);
	  printf("\n");
	  PrintNode(iname);
	}
      }
      break;
    case LOCAL: 
      {
	char netpasswd[PASSWDMAXLEN]="";
	printf("\n%s\n",LOCALstr);
	printf("New network password to set: ");
	fgets(netpasswd, PASSWDMAXLEN, stdin);
	netpasswd[strlen(netpasswd)-1]='\0';
	if(!DoMain(LOCAL,iname,netpasswd,NULL))
	  printf("OK\n");
      }
      break;
      case REMOTE: 
      {
	char netpasswd[PASSWDMAXLEN]="";
	char devpasswd[PASSWDMAXLEN]="";
	printf("\n%s\n",REMOTEstr);
	printf("New network password to set: ");
	fgets(netpasswd, PASSWDMAXLEN, stdin);
	netpasswd[strlen(netpasswd)-1]='\0';
	printf("Remote device password: ");
	fgets(devpasswd, PASSWDMAXLEN, stdin);
	devpasswd[strlen(devpasswd)-1]='\0';
	
	if(!DoMain(REMOTE,iname,netpasswd,devpasswd))
	  printf("OK\n");
	
      }
      break;
    }

  }

  return 0;
}


inline void CloseAll(int fd,pid_t pid){
  free(p_devTable);
  close(fd);
  kill(pid,15);
  wait(NULL);
}


/*
 *   Function that forks the processes
 */
int DoMain(char action,char* iname,char* newNek,char* remotePasswd)
{
  pid_t childpid;
  int retval=0;
  int mypipe[2];  
  int i=0;  
  unsigned char strPacket[MAX_PACK_LEN];  
  unsigned char nullMAC[MACSIZE]={0};
  int retry;
  char* myArgv[3];


  myArgv[0]="dlanson";
  myArgv[1]=iname;
  myArgv[2]=NULL;
  
  strcpy(localIfName,iname);
  
  /*
   *   Check to be root
   */
  if(getuid()!=0){
    printf("\nYou must be root to run dlanconfig tool.\n"); 
    printf("Bye Bye."); 
    return -1; 
  }

  if(CHeckFirst(iname))
    return -1;

  
  /*
   *   Create pipe
   */
  if(pipe(mypipe)!=0){
    printf("[ERROR]: DoMain: Error creating the pipe\n");
    strerror(errno);
    return -1;
  }


  /* 
   *  Vfork
   */
  if((childpid=fork())==0){ 

    close(mypipe[0]);
    dup2(mypipe[1],STDOUT_FILENO); 
    close(mypipe[1]);

    execvp(SON_PROCESS_NAME,(char**)myArgv);
    fprintf(stderr,"cf: dlanconfig_son return\n");
    strerror(errno);
    fflush(stdout);
  }
    
  close(mypipe[1]);



  // get Local MAC:  
  printf("\nPlease wait.");fflush(stdout);    
  retval=1;
  for(retry=MAXRETRY;retry  && retval;retry--){
    printf(".");fflush(stdout);
    for(i=0;i<POLL_RETRY;i++){
      retval=SendLocalParExt(iname);
      if(retval<0){
	printf("\nUnable to send packets!\n");
	CloseAll(mypipe[0],childpid);
	return 1;
      }
      sleep(INTER_FRAME_DELAY);
    }
    printf("."); fflush(stdout);
    if(!ListenForPackets(mypipe[0],strPacket,MAX_PACKET_TO_PARSE,WAIT_AFTER_NOACTIVITY) &&
       !IsSameMAC(nullMAC,p_devTable->locBridge.MAC)){
      printf(".");fflush(stdout);
      retval=0;
      break;			     
    }
  }
  if(retval){
    printf("\nUnable to get LocalMAC!\n ");
    CloseAll(mypipe[0],childpid);
    return -1;
  }
  
  // here if retval=0;
  switch(action){
  case GETLOCMAC:
    printf("\nLocal dLAN device has MAC address ");
    PrintMAC(p_devTable->locBridge.MAC);
    printf("\n");
    CloseAll(mypipe[0],childpid);
    return 0;
    break;
  case POLL: 
    retval=API_NetwPoll(iname);
    break;
  case LOCAL: 
    for(i=0;i<SET_LOC_NEK_RETRY;i++)
      retval+=SendSetLocalNek(iname, newNek); 
    break;
  case REMOTE:
    for(i=0;i<SET_REMOTE_NEK_RETRY;i++)
      retval+=SendSetRemoteNek(iname,newNek,remotePasswd);
    break; 
  default: 
    printf("[WARNING]: DoMain: unknown action required\n");  
    return 0;
    break;    
  }
 
  if(retval<=0){
    printf("Unable to send packets!\n");
    CloseAll(mypipe[0],childpid);
    return 1;
  }
  
  retval=ListenForPackets(mypipe[0],strPacket,MAX_PACKET_TO_PARSE,WAIT_AFTER_NOACTIVITY);


  if(retval==0 || retval==-1){    
    CloseAll(mypipe[0],childpid);
    return 0;
  }
  
  printf("Operation FAILED\n");  
  CloseAll(mypipe[0],childpid);
  return 1;
}


/*
 *   Wait until expectedBytes arrive
 */
int   WaitForNbytes(int fd,int expectedBytes,unsigned char* buf,struct timeval input_tv,int extraTimeSec){
  fd_set rfds;
  int arrivedBytes=0;
  int ready=0;
  struct timeval MyTimeval,tv; 

  FD_ZERO(&rfds);
  FD_SET(fd, &rfds);
  arrivedBytes=0;
  tv.tv_sec=input_tv.tv_sec;  
  tv.tv_usec=input_tv.tv_usec;
  
  ready=select(fd+1, &rfds, NULL, NULL, &tv);
  if(ready)   
    arrivedBytes=read(fd,buf,expectedBytes);
  else        
    return 0;
  
  if(arrivedBytes==expectedBytes)
    return 1;
  
  if(arrivedBytes==0){ 
    printf("[ERROR]: WaitForNbytes: no bytes to read\n");
    return -1;
  }
  
  if(arrivedBytes<expectedBytes) { 
    FD_ZERO(&rfds);
    FD_SET(fd, &rfds);
    MyTimeval.tv_sec=3;
    MyTimeval.tv_usec=0;  
    ready=select(fd+1, &rfds, NULL, NULL, &MyTimeval);
    if(ready)
      arrivedBytes+=read(fd,&(buf[arrivedBytes]),expectedBytes-arrivedBytes);
    if(arrivedBytes==expectedBytes) { 
      return(1); 
    }else{
      printf("[ERROR]: WaitForNbytes: no bytes to read\n");
      return(-1);
    }
  }
  printf("[ERROR]: WaitForNbytes: incomplete reading\n");
  return (-1); 
}


/*
 *   Get succesive packet
 */
int GetNextPacket(int fileDescr,void *strPacket)
{
  struct timeval tv;
  int retval=0;
  unsigned char strHeader[PACKET_HEADERSIZE]="";
  int packLen=0;
  
  tv.tv_sec=SELECT_TIMEOUT_S;
  tv.tv_usec=SELECT_TIMEOUT_US;
  
  retval=WaitForNbytes(fileDescr,PACKET_HEADERSIZE-1,strHeader,tv,EXTRA_WAIT_TIME_S);

  switch(retval){
  case -1:
    printf("[ERROR]: GetNextPacket: broken protocol, incomplete Header\n");
    return(-1);
  case 0:
    return 0;
  case 1:
    strHeader[PACKET_HEADERSIZE-1]='\0';
    packLen=atoi(strHeader);
    memset(strPacket,0,MAX_PACK_LEN);
    retval=WaitForNbytes(fileDescr,packLen,strPacket,tv,EXTRA_WAIT_TIME_S);
    
    if(retval==-1){
      printf("[ERROR]: GetNextPacket: broken protocol, incomplete packet!\n");
      return (-1);
    }
    
    if(retval==0) {
      printf("[ERROR]: GetNextPacket: broken protocol, header present but no packet available!\n");
      return(0);
    }   
    {
      /**********************/
      int c=0;
      struct API_ReqNetStatsResp* p;
      p=(struct API_ReqNetStatsResp*)strPacket;
      c++;
      /*******************/
    }
    return(1);
  }
  
  return -1;
}



/*
 *   Listen until new packets arrive
 */
int ListenForPackets(int fileDescr,char* buffer,int maxNumber,int maxTimeS)
{
  int receivedCount=0;
  int timeoutCount=0;
  int timeoutN=0;
  int retval=0;

  timeoutN=(1000000*maxTimeS)/SELECT_TIMEOUT_US;
  while(receivedCount<maxNumber && timeoutCount<timeoutN) {
    retval=GetNextPacket(fileDescr,buffer);
    switch(retval){
    case -1:
      printf("[ERROR]: ListenForPackets: get next packet = %d\n",retval );   
      return -1;
    case 0:
      timeoutCount++;  
      break;
    case 1:
      PacketReceive_callback(buffer); 
      receivedCount++;    
      break;
    }
  }
  
  return(0);

}


