/* 
   dlanconfig.h: dLANConfig tool for HomePlug devices for linux

   Copyrighted 2004 by devolo AG
   Only for use with MicroLink dLAN adaptors from devolo.

   Redistribution in source form, with or without modification,
   are NOT permitted.
   This program uses explicitly confidential information
   about HomePlug standard.
	    
*/

/*
 *   Include config.h that ./configure generates
 *   and check if all function are present
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#ifndef HAVE_DUP2
#  error "No dup2 function."
#endif
#ifndef HAVE_STRERROR
#  error "No strerror function."
#endif
#ifndef HAVE_SELECT
#  error "No select function."
#endif
#ifndef HAVE_MEMSET
#  error "No memset function."
#endif
#ifndef HAVE_SOCKET
#  error "No socket function."
#endif
#ifndef HAVE_VFORK
#  ifndef HAVE_FORK
#     error "No fork function."
#  else
#     define vfork fork
#  endif
#endif
#endif

/*
 *   Max password length
 */
#define PASSWDMAXLEN 56

/*
 *  HomePlug and MME specific constants
 */
#define PACKET_HEADERSIZE         0x5
#define ETHERTYPE_INTELLON	  0x887B  // Ethernet Type 
#define	CODE_ReqStats	          0x07    // Request Stats Command       
#define	CODE_ReqNetStats          0x1a	  // Request Net Stats Command   
#define CODE_SetLocPar            0x19    // Set Local Parameters
#define	CODE_ReqStatsResp         0x08	  // Request Stats Response	
#define	CODE_ReturnNetStats       0x80	  // Return Network Stats Cmd    
#define SOCK_PROTO                0x1000  // Protocol number

#define	MME_ReqStatsSize          0                                       
#define MME_LocalParExtSize       0xF       
#define MME_MIN_FRAME_LEN         64      
#define MME_NET_STAT_FRAME_LEN    204     
#define MME_HDR_LEN               14      

/*
 *  MAC and DEK size
 */
#define MACSIZE                   6       
#define DECKSIZE                  8       

/*
 *   Max number of remote devices and 
 *   local interfaces
 */
#define MAX_BRIDGED_DEVS          40      // Maximum number of device
#define MAX_NET_INTERFACE_NUM     16      

/*
 *  Command retry constant
 */
#define POLL_RETRY                1       
#define SET_LOC_NEK_RETRY         1       
#define SET_REMOTE_NEK_RETRY      1       

/*
 *  Delay and timeout constant
 */
#define INTER_FRAME_DELAY         1       
#define EXTRA_WAIT_TIME_S         1       
#define WAIT_AFTER_NOACTIVITY     1       
#define SELECT_TIMEOUT_S          0       
#define SELECT_TIMEOUT_US         100000  

/*
 *  Specific length
 */
#define MAX_PACK_LEN              1500
#define HDR_LEN                   4
#define MAX_PACKET_TO_PARSE       30      
#define MAX_INAME_LEN             32      

#define DUMP_UNKNOWN_INTELLON_FRAME 1 
#define DUMP_INFO_LEVEL           0             

/*
 *  Constants used in rate calculation
 */
#define	M_RATE                    ( (588.0 - 38.0) / 481.0 )
#define	B_RATE                    ( 14.0 - M_RATE * (519.0/42.0) )

/*
 * This struct describes our local interface
 */
typedef struct NetDev {
  int index;      
  char iname[MAX_INAME_LEN];
  unsigned char mac[MACSIZE];
}NetDev_t;

/*
 * This struct describes a generic device
 */
typedef struct devolo_node
{
  unsigned char MAC[MACSIZE]; 
  unsigned char DEK[DECKSIZE]; 
  unsigned char NEK[DECKSIZE];
  double rate;                
  short toggled;              
  unsigned short devType;     
} devolo_node_t;


/*
 * This struct contains all info we need
 * 
 */
typedef struct devolo_table{
  int devCounter;  
  struct devolo_node locBridge;
  struct devolo_node list[MAX_BRIDGED_DEVS];
}devolo_table_t;


/*
 *  APIs and specific structures
 */

int API_DoAction(int action,char* iname,char* newNek,char* remotePasswd);
struct devolo_table* API_Alloc_DevTable(void);
void API_PrintNode();

typedef struct API_NetW_DA{
  unsigned char	DA[MACSIZE]; 
  unsigned short  BYTES40;
  unsigned short 	FAILS;
  unsigned short 	DROPS;
} __attribute__ ((packed)) API_NetW_DA_t;

#define NWSIZE   15

typedef struct API_ReqNetStatsResp{
  unsigned char   DA[MACSIZE]; 
  unsigned char   SA[MACSIZE];	
  unsigned short  Ethertype;
  unsigned char	  MCTRL;	
  unsigned char	  MEHDR;
  unsigned char	  MELEN;
  unsigned char	  NETW_CTRL;
  unsigned short  BYTES40_ROBO;
  unsigned short  FAILS_ROBO;
  unsigned short  DROPS_ROBO;
  API_NetW_DA_t	  NW[NWSIZE];
}  __attribute__ ((packed))API_ReqNetStatsResp_t;


/*
 *   Specific MME structures
 */
typedef struct MME_PackedHdr{
  unsigned char   DA[MACSIZE];
  unsigned char   SA[MACSIZE];
  unsigned short  Ethertype;  
  unsigned char	  MCTRL ;     
  unsigned char	  MEHDR;      
  unsigned char	  MELEN;      
  unsigned char   NETW_CTRL;  
} __attribute__ ((packed))MME_PackedHdr_t;


typedef struct mme_hdr{
  unsigned char	  MCTRL ;
  unsigned char	  MEHDR; 
  unsigned char	  MELEN; 
  unsigned char   NETW_CTRL;
} __attribute__ ((packed))mme_hdr_t;


typedef struct mme_locParExtReq{
  unsigned char	  MCTRL ;         
  unsigned char	  MAC[MACSIZE];   
  unsigned char	  DECK[DECKSIZE]; 
  unsigned char   NETW_CTRL;      
} __attribute__ ((packed))mme_locParExtReq_t;


typedef struct mme_locParExtResp 
{
  unsigned char   DA[MACSIZE]; 
  unsigned char   SA[MACSIZE];	
  unsigned short  Ethertype;
  unsigned char	  MCTRL; 	
  unsigned char	  MEHDR;	
  unsigned char	  MELEN;
  unsigned char	  CTRL ;         
  unsigned char	  MAC[MACSIZE]; 
  unsigned char	  DEK[DECKSIZE];
  unsigned char   EKS;          
  unsigned char   NEK[DECKSIZE];
  unsigned char   TXCHAR[3];    
  unsigned char   OVERRIDES[22];
  unsigned char   RST_CNT[2];   
} __attribute__ ((packed))mme_locParExtResp_t;



typedef struct mme_double{ 
  unsigned char	  CTRL;  
  unsigned char	  HDR1;	 
  unsigned char	  LEN1;  
  unsigned char   EKS;   
  unsigned char   NEK[DECKSIZE];
  unsigned char	  HDR2;	        
  unsigned char	  LEN2;         
  unsigned char   TXCHAR[3];    
} __attribute__ ((packed))mme_double_t;



typedef struct mme_triple{
  unsigned char	  CTRL;  
  unsigned char	  HDR1;	 
  unsigned char	  LEN1;  
  unsigned char   EKS1;  
  unsigned char   NEK1[DECKSIZE];
  unsigned char	  HDR2;	         
  unsigned char	  LEN2;          
  unsigned char   EKS2;          
  unsigned char   NEK2[DECKSIZE];
  unsigned char	  HDR3;	         
  unsigned char	  LEN3;          
  unsigned char   TXCHAR[3];     
} __attribute__ ((packed))mme_triple_t;

